@@ -23,8 +23,12 @@
 #ifndef __YELP_SCROLLKEEPER_H__
 #define __YELP_SCROLLKEEPER_H__
 
+#include <glib.h>
 #include <gtk/gtktreestore.h>
 
-gboolean yelp_scrollkeeper_init (GtkTreeStore *store);
+gboolean       yelp_scrollkeeper_init               (GNode          *tree);
+GNode *        yelp_scrollkeeper_lookup_seriesid    (const gchar    *seriesid);
+
+GNode *        yelp_scrollkeeper_get_toc_tree       (const gchar    *docpath);
 
 #endif /* __YELP_SCROLLKEEPER_H__ */
